﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Data;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization.Operators.Crossovers {
  [Item("MultiIntValueCrossover", "Randomly selects and applies one of its crossovers every time it is called.")]
  [StorableClass]
  public class MultiIntValueCrossover : StochasticMultiBranch<IIntValueCrossover>, IIntValueCrossover, IStochasticOperator {
    public override bool CanChangeName {
      get { return false; }
    }
    protected override bool CreateChildOperation {
      get { return true; }
    }

    [StorableConstructor]
    protected MultiIntValueCrossover(bool deserializing) : base(deserializing) { }
    public MultiIntValueCrossover() {
      foreach (Type type in ApplicationManager.Manager.GetTypes(typeof(IIntValueCrossover)).OrderBy(op => op.Name)) {
        if (!typeof(MultiOperator<IIntValueCrossover>).IsAssignableFrom(type))
          Operators.Add((IIntValueCrossover)Activator.CreateInstance(type), true);
      }
    }
    protected MultiIntValueCrossover(MultiIntValueCrossover original, Cloner cloner) : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiIntValueCrossover(this, cloner);
    }

    public override IOperation Apply() {
      if (Operators.Count == 0) throw new InvalidOperationException(Name + ": Please add at least one permutation crossover to choose from.");
      return base.Apply();
    }

    public void Apply(IRandom random, IntValue value, IntValue other, IntValueRange range) {

    }
  }
}
